import numpy as np
import pandas as pd
from prostate_cancer import simulate_prostate_cancer_data
import matplotlib.pylab as plt
from matplotlib import pyplot
from sklearn.model_selection import train_test_split, KFold

from sklearn.linear_model import LogisticRegression
from sklearn.neighbors import KNeighborsClassifier
from sklearn.svm import SVC
from sklearn.tree import DecisionTreeClassifier

from sklearn.metrics import roc_curve
from sklearn.metrics import roc_auc_score

X,y = simulate_prostate_cancer_data()

y = y[:,0]

### confusion matrix ###
from sklearn.metrics import confusion_matrix
X_train, X_test, y_train, y_test = \
train_test_split(X, y, test_size=0.5, random_state=100)
lr_model = LogisticRegression(solver="liblinear")
lr_model.fit(X_train, y_train)
y_test_pred = lr_model.predict(X_test)
cm = confusion_matrix(y_test, y_test_pred)

cm_df = pd.DataFrame(cm)
print(cm_df.to_latex(index=True))

### ROC ###
y_test_proba = lr_model.predict_proba(X_test)
fpr, tpr, thresholds = roc_curve(y_test, y_test_proba[:,1])
auc = roc_auc_score(y_test, y_test_proba[:,1])

fig = plt.figure()
ax = fig.add_subplot(111)
ax.plot([0, 1], [0, 1], linestyle='--')
ax.plot(fpr, tpr, marker='.')
ax.text(0.7, 0.3, 'AUC: '+str(round(auc,2)))
pyplot.xlabel("False Positive Rate")
pyplot.ylabel("True Positive Rate")
pyplot.savefig('../figures/prostate_cancer_lr_roc.eps')

### get latex ROC table 
roc_table = np.append(tpr[:,np.newaxis], fpr[:,np.newaxis], axis=1)
roc_table = np.append(roc_table, thresholds[:,np.newaxis], axis=1)
roc_df = pd.DataFrame(roc_table, columns=['TPR', 'FPR', 'Threshold'])
print(roc_df.to_latex(index=False))

### cross validation ###
def cross_validate(model, X_input, Y_output):
    kf = KFold(n_splits=5, random_state=1, shuffle=True)
    true_labels = np.array([0] * X_input.shape[0])
    pred_scores = np.array([0.0] * X_input.shape[0])
    for train_index, test_index in kf.split(X_input):
        model.fit(X_input[train_index], Y_output[train_index])
        true_labels[test_index] = Y_output[test_index]
        pred_scores[test_index] = model.predict_proba(X_input[test_index])[:,1]
    return (true_labels, pred_scores)

true_labels,pred_scores = cross_validate(lr_model, X, y)

fpr, tpr, thresholds = roc_curve(true_labels, pred_scores)
auc = roc_auc_score(true_labels, pred_scores)

fig = plt.figure()
ax = fig.add_subplot(111)
ax.plot([0, 1], [0, 1], linestyle='--')
# plot the roc curve for the model
ax.plot(fpr, tpr, marker='.')

ax.text(0.7, 0.3, 'AUC: '+str(round(auc,2)))

pyplot.xlabel("False Positive Rate")
pyplot.ylabel("True Positive Rate")
pyplot.savefig('../figures/prostate_cancer_lr_crossvalid_roc.eps')


### Method comparison ###
models = [LogisticRegression(solver="liblinear"),
         KNeighborsClassifier(),
         SVC(probability=True, gamma='auto'),
         DecisionTreeClassifier()]

perf = {}

for model in models:
    model_name = type(model).__name__
    print(model_name)
    label,pred = cross_validate(model, X, y)
    fpr, tpr, thresholds = roc_curve(label, pred)
    auc = roc_auc_score(label, pred)
    perf[model_name] = {'fpr':fpr,'tpr':tpr,'auc':auc}    


fig = plt.figure()
ax = fig.add_subplot(111)
ax.plot([0, 1], [0, 1], linestyle='--')

model_abbrev = {"LogisticRegression":"LR",
               "KNeighborsClassifier":"KNN",
               "SVC":"SVC",
               "DecisionTreeClassifier":"DT"}

i = 0
for model_name, model_perf in perf.items():    
    mymodel = model_abbrev[model_name]
    ax.plot(model_perf['fpr'], model_perf['tpr'], 
            label=mymodel)
    ax.text(0.5, i, mymodel + ': AUC = '+str(round(model_perf['auc'],2)))
    i += 0.1

pyplot.legend(loc='best')    
pyplot.xlabel("False Positive Rate")
pyplot.ylabel("True Positive Rate")
pyplot.savefig('../figures/roc_multimethods.eps')





























